import re
from flask import g
from nameko.standalone.rpc import ClusterRpcProxy
from .connection_pool import ConnectionPool
from .errors import (
    BadConfigurationError,
    ClusterNotConfiguredError
)

class PooledClusterRpcProxy(object):

    _pool = None
    _config = None

    def __init__(self, config=None):
        if config:
            self.configure(config)

    def configure(self, config):
        if not config.get('AMQP_URI'):
            raise BadConfigurationError("Please provide a valid configuration.")

        self._config = config
        self._pool = ConnectionPool(
            self._get_nameko_connection,
            initial_connections=config.get('INITIAL_CONNECTIONS', 2),
            max_connections=config.get('MAX_CONNECTIONS', 8)
        )

    def _get_nameko_connection(self):
        proxy = ClusterRpcProxy(self._config)
        return proxy.start()

    def get_connection(self):
        if not self._pool:
            raise ClusterNotConfiguredError("Please configure your cluster beore requesting a connection.")
        return self._pool.get_connection()

    def release_connection(self, connection):
        return self._pool.release_connection(connection)

class LazyServiceProxy(object):
    def __init__(self, get_connection, service):
        self.get_connection = get_connection
        self.service = service
    def __getattr__(self, name):
        return getattr(getattr(self.get_connection(), self.service), name)

class FlaskPooledClusterRpcProxy(PooledClusterRpcProxy):
    def __init__(self, app=None, connect_on_method_call=True):
        self._connect_on_method_call = connect_on_method_call
        if app:
            self.init_app(app)

    def init_app(self, app):
        config = dict()
        for key, val in app.config.iteritems():
            match = re.match(r"NAMEKO\_(?P<name>.*)", key)
            if match:
                config[match.group('name')] = val
        self.configure(config)

        self._connect_on_method_call = config.get('NAMEKO_CONNECT_ON_METHOD_CALL', self._connect_on_method_call)

        app.teardown_appcontext(self._teardown_nameko_connection)

    def get_connection(self):
        connection = getattr(g, '_nameko_connection', None)
        if not connection:
            connection = super(FlaskPooledClusterRpcProxy, self).get_connection()
            g._nameko_connection = connection
        return connection

    def _teardown_nameko_connection(self, exception):
        connection = getattr(g, '_nameko_connection', None)
        if connection is not None:
            self.release_connection(connection)

    def _get_service(self, service):
        if self._connect_on_method_call:
            return LazyServiceProxy(lambda: self.get_connection(), service)
        else:
            return getattr(self.get_connection(), service)

    def __getattr__(self, name):
        return self._get_service(name)

    def __getitem__(self, name):
        return self._get_service(name)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
import pytest
from mock import Mock
from flask_nameko.connection_pool import ConnectionPool
from flask_nameko.errors import ClientUnavailableError

@pytest.fixture
def some_fixture():
    pass

@pytest.fixture
def get_connection():
    connection = Mock(side_effect=lambda: object())
    return connection

def test_connections_recycled(get_connection):
    pool = ConnectionPool(get_connection, initial_connections=0)

    o = pool.get_connection()
    pool.release_connection(o)
    o1 = pool.get_connection()
    o2 = pool.get_connection()

    assert o1 == o
    assert o1 != o2

def test_new_connections_used(get_connection):
    pool = ConnectionPool(get_connection, initial_connections=0)

    o = pool.get_connection()
    o1 = pool.get_connection()

    assert o1 != o

def test_max_connections_raises(get_connection):
    pool = ConnectionPool(get_connection, initial_connections=0, max_connections=2)

    pool.get_connection()
    pool.get_connection()

    with pytest.raises(ClientUnavailableError):
        pool.get_connection(next_timeout=0)

def test_creates_initial_connections(get_connection):
    pool = ConnectionPool(get_connection, initial_connections=2)
    assert get_connection.call_count == 2



#!/usr/bin/env python
# -*- coding: utf-8 -*-


try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


with open('README.md') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [
    # TODO: put package requirements here
    #
    # These requirements should be pinned according to SemVer.
    # i.e.
    # "left_pad>=1.0, <2.0"
    "nameko",
    "flask"
]

test_requirements = [
    # TODO: put package test requirements here
    "mock==2.0.0",
    "pytest"
]

setup(
    name='flask_nameko',
    version='1.1.1',
    description="A wrapper for using nameko services with Flask",
    long_description=readme + '\n\n' + history,
    author="Jesse Pollak",
    author_email='jesse@getclef.com',
    url='https://github.com/clef/flask_nameko',
    packages=[
        'flask_nameko',
    ],
    package_dir={'flask_nameko':
                 'flask_nameko'},
    include_package_data=True,
    install_requires=requirements,
    license="ISCL",
    zip_safe=False,
    keywords='flask_nameko',
    classifiers=[],
    test_suite='tests',
    tests_require=test_requirements,
    setup_requires=['pytest-runner']
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import pytest
from mock import Mock, patch
from flask import Flask, g
from flask_nameko import FlaskPooledClusterRpcProxy
from flask_nameko.proxies import LazyServiceProxy
from flask_nameko.errors import (
    ClientUnavailableError,
    ClusterNotConfiguredError
)

@pytest.fixture
def some_fixture():
    pass

@pytest.fixture
def flask_app():
    app = Flask(__name__)
    app.config.update(dict(NAMEKO_AMQP_URI='test'))
    return app

def test_configuration_pulls_nameko_names(flask_app):
    with patch.object(FlaskPooledClusterRpcProxy, 'configure') as configure:
        rpc = FlaskPooledClusterRpcProxy(flask_app)
        configure.assert_called_once_with(dict(AMQP_URI='test'))

def test_not_configured_raises_exception(flask_app):
    rpc = FlaskPooledClusterRpcProxy()
    with flask_app.test_request_context():
        with pytest.raises(ClusterNotConfiguredError):
            rpc.not_configured_service.test()

def test_connection_is_reused_in_app_context(flask_app):
    with patch('flask_nameko.proxies.ClusterRpcProxy'):
        rpc = FlaskPooledClusterRpcProxy(flask_app, connect_on_method_call=False)
        with flask_app.test_request_context():
            connection = rpc.get_connection()
            connection1 = rpc.get_connection()
            assert connection == connection1

def test_connection_is_returned_and_reused_when_app_context_ends(flask_app):
    flask_app.config.update(dict(NAMEKO_INITIAL_CONNECTIONS=1))
    with patch('flask_nameko.proxies.ClusterRpcProxy'):
        rpc = FlaskPooledClusterRpcProxy(flask_app, connect_on_method_call=False)
        with flask_app.test_request_context():
            connection = rpc.get_connection()
        with flask_app.test_request_context():
            connection1 = rpc.get_connection()
        assert connection1 ==  connection

def test_new_connection_is_used_for_new_app_context(flask_app):
    with patch('flask_nameko.proxies.ClusterRpcProxy', side_effect=lambda x: Mock()):
        rpc = FlaskPooledClusterRpcProxy(flask_app, connect_on_method_call=False)

        with flask_app.test_request_context():
            connection = rpc.get_connection()

        with flask_app.test_request_context():
            connection1 = rpc.get_connection()

        assert connection1 != connection

def test_connect_on_method_call_false_returns_connection(flask_app):
    with flask_app.test_request_context():
        with patch('flask_nameko.proxies.ClusterRpcProxy', side_effect=lambda x: Mock()):
            rpc = FlaskPooledClusterRpcProxy(flask_app, connect_on_method_call=False)
            assert isinstance(rpc.service, Mock)

def test_connect_on_method_call_returns_lazy_proxy(flask_app):
    with patch('flask_nameko.proxies.ClusterRpcProxy', side_effect=lambda x: Mock()):
        rpc = FlaskPooledClusterRpcProxy(flask_app, connect_on_method_call=True)
        assert isinstance(rpc.service, LazyServiceProxy)



from Queue import Queue, Empty
from threading import Lock
from .errors import ClientUnavailableError

class ConnectionPool(object):
    def __init__(self, get_connection, initial_connections=2, max_connections=8):
        """
        Create a new pool
        :param func get_connection: The function that returns a connection
        :param int initial_connections: The initial number of connection objects to create
        :param int max_connections: The maximum amount of connections to create. These
          connections will only be created on demand and will potentially be
          destroyed once they have been returned via a call to
          :meth:`release_connection`
        constructor
        """
        self._get_connection = get_connection
        self._queue = Queue()
        self._current_connections = 0
        self._max_connections = max_connections
        self._lock = Lock()

        for x in range(initial_connections):
            connection = self._make_connection()
            self._queue.put(connection)

    def _make_connection(self):
        ret = self._get_connection()
        self._current_connections += 1
        return ret

    def get_connection(self, initial_timeout=0.05, next_timeout=1):
        """
        Wait until a connection instance is available
        :param float initial_timeout:
          how long to wait initially for an existing connection to complete
        :param float next_timeout:
          if the pool could not obtain a connection during the initial timeout,
          and we have allocated the maximum available number of connections, wait
          this long until we can retrieve another one
        :return: A connection object
        """
        try:
            return self._queue.get(True, initial_timeout)
        except Empty:
            try:
                self._lock.acquire()
                if self._current_connections == self._max_connections:
                    raise ClientUnavailableError("Too many connections in use")
                cb = self._make_connection()
                return cb
            except ClientUnavailableError as ex:
                try:
                    return self._queue.get(True, next_timeout)
                except Empty:
                    raise ex
            finally:
                self._lock.release()

    def release_connection(self, cb):
        """
        Return a Connection object to the pool
        :param Connection cb: the connection to release
        """
        self._queue.put(cb, True)
